@@ -2,11 +2,11 @@ context("Checking sentiment")
 
 test_that("sentiment produces a data.table with numeric sentiment column",{
 
-    mytext <- c(
+    mytext <- get_sentences(c(
        'do you like it?  But I hate really bad dogs',
        'I am the best friend.',
        'Do you really like it?  I\'m not a fan'
-    )
+    ))
     expect_true(is(sentiment(mytext), "data.table"))
     expect_equal(nrow(sentiment(mytext)), 5)
 
@@ -17,11 +17,11 @@ test_that("sentiment produces a data.table with numeric sentiment column",{
 
 test_that("sentiment question.weight = 0 alters the output",{
 
-    mytext <- c(
+    mytext <- get_sentences(c(
        'do you like it?  But I hate really bad dogs',
        'I am the best friend.',
        'Do you really like it?  I\'m not a fan'
-    )
+    ))
     x <- sentiment(mytext)[["sentiment"]]
     y <- sentiment(mytext, question.weight = 0)[["sentiment"]]
 
@@ -32,11 +32,11 @@ test_that("sentiment question.weight = 0 alters the output",{
 
 test_that("sentiment n.before/n.after alters the output",{
 
-    mytext <- c(
+    mytext <- get_sentences(c(
        'do you like it?  But I hate really bad dogs',
        'I am the best friend.',
        'Do you really like it?  I\'m not a fan'
-    )
+    ))
     x <- sentiment(mytext)[["sentiment"]]
     y <- sentiment(mytext, n.before = Inf, n.after = 0)[["sentiment"]]
 
@@ -45,11 +45,11 @@ test_that("sentiment n.before/n.after alters the output",{
 
 test_that("sentiment adversative.weight = 0 alters the output",{
 
-    mytext <- c(
-       'do you like it?  But I hate really bad dogs',
+    mytext <- get_sentences(c(
+       'do you like it, but I hate really bad dogs',
        'I am the best friend.',
-       'I like it but I really like eggs'
-    )
+       'Do you really like it?  I\'m not a fan'
+    ))
     x <- sentiment(mytext)[["sentiment"]]
     y <- sentiment(mytext, adversative.weight=3)[["sentiment"]]
 
@@ -59,11 +59,11 @@ test_that("sentiment adversative.weight = 0 alters the output",{
 
 test_that("sentiment_by plots a ggplot object",{
 
-    mytext <- c(
+    mytext <- get_sentences(c(
        'do you like it?  But I hate really bad dogs',
        'I am the best friend.',
-       'Do you really like it?  I\'m not happy'
-    )
+       'Do you really like it?  I\'m not a fan'
+    ))
 
     expect_true(is(plot(sentiment(mytext)), "ggplot"))
 
